# Generated by Django 2.0.7 on 2018-08-02 12:55

from django.db import migrations, models
import django.db.models.deletion
import easy_thumbnails.fields
import tango_shared.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
    ]

    operations = [
        migrations.CreateModel(
            name='Gallery',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('overline', models.CharField(blank=True, help_text='A short headline over the main headline.', max_length=200, null=True, verbose_name='Kicker/Overline')),
                ('title', models.CharField(help_text='The title for this content.', max_length=200, verbose_name='Title/Headline')),
                ('subhead', models.CharField(blank=True, help_text='A short extra headline below the main headline.', max_length=200, verbose_name='Subhead/Deck')),
                ('slug', models.SlugField(help_text='Used for URLs and identification.\n        Will auto-fill, but can be edited with caution.\n        ', max_length=200)),
                ('summary', models.TextField(blank=True, help_text="You should summarize the content.\n        It's better for search engines, and for people browsing lists of content.\n        If you don't, a summary will be created. But you should.\n        ", verbose_name='Summary description')),
                ('summary_formatted', models.TextField(blank=True, editable=False, help_text='Stores HTML formatted, sanitized version of summary')),
                ('featured', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('enable_comments', models.BooleanField(default=True)),
                ('last_modified', models.DateTimeField(auto_now=True)),
                ('has_image', models.BooleanField(default=False, editable=False, max_length=200)),
                ('credit', models.CharField(blank=True, max_length=200)),
                ('published', models.BooleanField(default=True)),
                ('sites', models.ManyToManyField(default=[1], to='sites.Site')),
            ],
            options={
                'verbose_name_plural': 'galleries',
            },
        ),
        migrations.CreateModel(
            name='GalleryImage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('image', easy_thumbnails.fields.ThumbnailerImageField(blank=True, help_text='Size should be a minimum of 720px and no more than 2000px high or wide.', upload_to=tango_shared.models.set_img_path)),
                ('caption', models.CharField(blank=True, max_length=255, null=True)),
                ('byline', models.CharField(blank=True, max_length=200, null=True)),
                ('credit', models.CharField(blank=True, max_length=200, null=True, verbose_name='Credit/source')),
                ('order', models.IntegerField(blank=True, help_text='For manual sorting.', null=True)),
                ('thumb', models.CharField(blank=True, editable=False, max_length=200, null=True)),
                ('is_vertical', models.BooleanField(default=False, editable=False)),
                ('gallery', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='photos.Gallery')),
            ],
            options={
                'ordering': ['order', '-id'],
                'abstract': False,
            },
        ),
    ]



import datetime

from django.conf import settings
from django.db import models


RESTRICT_CONTENT_TO_SITE = getattr(settings, 'RESTRICT_CONTENT_TO_SITE', False)
now = datetime.datetime.utcnow()


class GalleryManager(models.Manager):
    """
    Custom gallery objects manager.
    If RESTRICT_CONTENT_TO_SITE is True in settings,
    will limit galleries to current site.

    Usage is simply gallery.objects.all()
    """
    def get_queryset(self):
        galleries = super(GalleryManager, self).get_queryset()
        if RESTRICT_CONTENT_TO_SITE:
            galleries.filter(sites__id__exact=settings.SITE_ID)
        return galleries


class PublishedGalleryManager(GalleryManager):
    """
    Extends GalleryManager to only return galleries
    - That are published
    - With a pub_date greater than or equal to now.

    Usage is gallery.published.all()
    """
    def get_queryset(self):
        galleries = super(PublishedGalleryManager, self).get_queryset()
        galleries = galleries.filter(published=True, created__lte=now)
        return galleries



from django import template
from django.conf import settings

from photos.models import Gallery

register = template.Library()


@register.simple_tag()
def get_galleries(count=5):
    """
    Returns one or more galleries based on num as an object_list.
    Usage:
    {% get_galleries <NUM> as galleries %}
    """
    return Gallery.published.all()[:count]


@register.inclusion_tag('photos/includes/related_galleries.html')
def get_related_galleries(gallery, count=5):
    """
    Gets latest related galleries from same section as originating gallery.

    Count defaults to five but can be overridden.

    Usage: {% get_related_galleries gallery <10> %}
    """
    # just get the first cat. If they assigned to more than one, tough
    try:
        cat = gallery.sections.all()[0]
        related = cat.gallery_categories.filter(published=True).exclude(id=gallery.id).order_by('-id')[:count]
    except:
        related = None
    return {'related': related, 'MEDIA_URL': settings.MEDIA_URL}



import unittest

from django.template import Template, Context
from django.test import TestCase
from django.urls import reverse

from .admin import GalleryAdmin
from .models import Gallery, GalleryImage


class TestGalleries(TestCase):
    fixtures = ['photos.json']

    def setUp(self):
        self.gallery = Gallery.objects.get(id=1)
        self.galleryImage = GalleryImage.objects.latest('id')

    def test_list(self):
        """
        Test gallery list
        """
        response = self.client.get(reverse('gallery_list'))
        self.assertEqual(response.status_code, 200)
        self.assertTrue('object_list' in response.context)
        self.assertTemplateUsed('photos/gallery_list.html')

    def test_gallery_detail(self):
        """
        Test for valid gallery detail.
        """
        response = self.client.get(reverse('gallery_detail', args=[self.gallery.slug]))
        self.assertEqual(response.status_code, 200)
        self.assertTrue('object' in response.context)
        self.assertEquals(self.gallery.id, response.context['object'].id)
        self.assertTemplateUsed('photos/gallery_detail.html')

    def test_gallery_model(self):
        self.assertTrue(self.gallery.has_image)
        self.assertEqual(self.gallery.get_image().image.url, self.galleryImage.image.url)
        self.gallery.galleryimage_set.all().delete()
        self.assertEquals(self.gallery.get_image(), None)

    @unittest.skip("Strange DeferredAttribute Error")
    def test_get_galleries_tag(self):
        "Test get galleries tag"
        out = Template(
            "{% load gallery_tags %}"
            "{% get_galleries 1 as galleries %}"
            "{% for gallery in galleries %}"
            "{{ gallery.title }}"
            "{% endfor %}"
        ).render(Context())
        self.assertEqual(out, self.gallery.title)

    def test_get_related_galleries_tag(self):
        """
        Test get related galleries.
        We have no related galleries in the json, so this should be short.
        """
        out = Template(
            "{% load gallery_tags %}"
            "{% get_related_galleries object 10 %}"
        ).render(Context({'object': self.gallery}))
        self.assertEqual(out, '\n')

    def test_admin_actions(self):
        gallery_actions = [action.short_description for action in GalleryAdmin.actions]
        self.assertTrue('Publish' in gallery_actions)
        self.assertTrue('Unpublish' in gallery_actions)



from django import forms

from .models import Gallery


class GalleryForm(forms.ModelForm):
    """
    Base form for gallery uploads
    """
    bulk_upload = forms.FileField(
        widget=forms.FileInput(attrs={'multiple': 'multiple'}),
        required=False
    )

    class Meta:
        model = Gallery
        fields = ['overline', 'title', 'slug', 'credit', 'summary', 'published', 'bulk_upload']
 


from django.urls import path
from django.views.generic import DetailView, ListView

from .models import Gallery

galleries = Gallery.published_objects.all()

urlpatterns = [
    path('', ListView.as_view(queryset=galleries), name="gallery_list"),
    path(
        '<slug:slug>/', 
        DetailView.as_view(
            queryset=galleries,
            template_name="photos/gallery_detail.html"
        ),
        name="gallery_detail"
    ),
]



# -*- coding: utf-8 -*-
from distutils.core import setup
from setuptools import find_packages

with open('docs/requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name='tango-photos',
    version='0.5.1',
    author=u'Tim Baxter',
    author_email='mail.baxter@gmail.com',
    url='http://github.com/tBaxter/tango-photos',
    license='LICENSE.txt',
    description='Reusable Django photo galleries app. Can be used with or without Tango.',
    long_description=open('README.md').read(),
    packages=find_packages(),
    install_requires=required,
    zip_safe=False,
    include_package_data=True
)



from django.contrib import admin

from .forms import GalleryForm
from .models import Gallery, GalleryImage


##### Actions
def make_published(self, request, queryset):
    rows_updated = queryset.update(published=True)
    if rows_updated == 1:
        message_bit = "1 gallery was"
    else:
        message_bit = "{} galleries were".format(rows_updated)
    self.message_user(request, "{} published.".format(message_bit))
make_published.short_description = "Publish"


def make_unpublished(self, request, queryset):
    rows_updated = queryset.update(published=False)
    if rows_updated == 1:
        message_bit = "1 gallery was"
    else:
        message_bit = "{} galleries were".format(rows_updated)
    self.message_user(request, "{} unpublished.".format(message_bit))
make_unpublished.short_description = "Unpublish"


class GalleryInline(admin.TabularInline):
    model = GalleryImage
    extra = 10
    fields = ('image', 'caption', 'byline', 'credit', 'order')


class GalleryAdmin(admin.ModelAdmin):
    form = GalleryForm

    ordering = ['-created']
    actions = [make_published, make_unpublished]

    list_display = ('title', 'credit', 'published', 'created',)
    list_filter = ['published',]
    prepopulated_fields = {'slug': ('title',)}
    search_fields = ['title', 'summary']

    inlines = [
        #BulkUploadInline,
        GalleryInline,
    ]
    fieldsets = (
        ('Header', {'fields': ('overline', 'title')}),
        ('Content', {'fields': ('summary', 'bulk_upload')}),
        ('Meta', {'fields': ('credit', 'published')}),
        ('Admin fields', {
            'description': 'You should rarely, if ever, need to touch these fields.',
            'fields': ('slug',),
            'classes': ['collapse']
        }),
    )

    def save_model(self, request, obj, form, change):
        obj.save()
        for img in request.FILES.getlist('bulk_upload'):
            GalleryImage(
                image = img,
                gallery = obj
            ).save()

        obj.bulk_upload = None
        obj.save()


class GalleryImageAdmin(admin.ModelAdmin):
    list_display = ('admin_thumb', 'gallery', 'caption', 'byline', )
    search_fields = ['gallery__title', 'caption', 'byline']
    list_filter = ('gallery',)


admin.site.register(Gallery, GalleryAdmin)
admin.site.register(GalleryImage, GalleryImageAdmin)



from django.db import models
from django.urls import reverse

from .managers import GalleryManager, PublishedGalleryManager
from tango_shared.models import ContentImage, BaseContentModel


class Gallery(BaseContentModel):
    """
    Allows for Gallery creation.
    If you get a "413 Entity Too Large" error when bulk uploading, adjust the Nginx configuration. 

    """
    credit = models.CharField(max_length=200, blank=True)
    published = models.BooleanField(default=True)

    # Managers
    objects = GalleryManager()
    published_objects = PublishedGalleryManager()

    class Meta:
        verbose_name_plural = "galleries"

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse('gallery_detail', args=[self.slug])

    def get_image(self):
        try:
            return self.galleryimage_set.all()[0]
        except IndexError:
            return None


class GalleryImage(ContentImage):
    gallery = models.ForeignKey(
        Gallery,
        on_delete=models.CASCADE
    )
